@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 8769 2007-11-08T05:55:36.931222Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 8884 2007-11-09T11:12:49.102360Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -11,7 +11,7 @@ use POE;
 use POE::Component::Client::Keepalive;
 use POE::Component::Client::HTTP;
 
-__PACKAGE__->mk_accessors($_) for qw(alias loop_alarm loop_delay resolver);
+__PACKAGE__->mk_accessors($_) for qw(alias loop_alarm loop_delay resolver clients);
 
 use constant UserAgentAlias => 'Gungho_Engine_POE_UserAgent_Alias';
 use constant DnsResolverAlias => 'Gungho_Engine_POE_DnsResolver_Alias';
@@ -56,10 +56,20 @@ sub run
     my %config = %{ $self->config || {} };
 
     my $keepalive_config = delete $config{keepalive} || {};
-    $keepalive_config->{keep_alive}   ||= 10;
-    $keepalive_config->{max_open}     ||= 200;
-    $keepalive_config->{max_per_host} ||= 5;
-    $keepalive_config->{timeout}      ||= 10;
+
+    {
+        my %defaults = (
+            keep_alive   => 10,
+            max_open     => 200,
+            max_per_host => 5,
+            timeout      => 10
+        );
+        while (my($key, $value) = each %defaults) {
+            if (! defined $keepalive_config->{$key}) {
+                $keepalive_config->{$key} = $value;
+            }
+        }
+    }
 
     my $keepalive = POE::Component::Client::Keepalive->new(%$keepalive_config);
 
@@ -97,14 +107,25 @@ sub run
         $self->resolver($resolver);
     }
 
-    POE::Component::Client::HTTP->spawn(
-        FollowRedirects   => 1,
-        Agent             => $c->user_agent,
-        Timeout           => 60,
-        %$client_config,
-        Alias             => &UserAgentAlias,
-        ConnectionManager => $keepalive,
-    );
+    # Oh, guess what. We will create as many clients as we were requested,
+    # just so that PoCo::Client::HTTP doesn't stall on us (as of 
+    # PoCo::Client::HTTP 0.82, PoCo::Client::HTTP tended to get filled up
+    # pretty quickcly)
+    $self->clients( [] );
+    my $spawn = $client_config->{spawn} || 2;
+    if ($spawn < 1) { $spawn = 2 }
+    for my $i ( 1 .. $spawn ) {
+        my $alias = join('-', &UserAgentAlias, $i);
+        push @{ $self->clients }, $alias;
+        POE::Component::Client::HTTP->spawn(
+            FollowRedirects   => 1,
+            Agent             => $c->user_agent,
+            Timeout           => 60,
+            %$client_config,
+            Alias             => $alias,
+            ConnectionManager => $keepalive,
+        );
+    }
 
     POE::Session->create(
         heap => { CONTEXT => $c },
@@ -130,12 +151,14 @@ sub _poe_session_start
 sub _poe_session_stop
 {
     $_[KERNEL]->alias_remove( $_[OBJECT]->alias );
+    eval {
+        $_[KERNEL]->post($_, 'shutdown') for @{ $_[OBJECT]->clients }
+    };
 }
 
 sub _poe_session_loop
 {
     my ($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
-    $self->loop_alarm(undef);
 
     my $c = $heap->{CONTEXT};
 
@@ -147,13 +170,16 @@ sub _poe_session_loop
     $c->dispatch_requests();
 
     my $alarm_id = $self->loop_alarm;
-    if (! $alarm_id) {
-        my $delay = $self->loop_delay;
-        if (! defined $delay || $delay <= 0) {
-            $delay = 1;
-        }
-        $self->loop_alarm($kernel->delay_set('session_loop', $delay));
+    if ($alarm_id) {
+        $kernel->alarm_remove( $alarm_id );
     }
+    my $delay = $self->loop_delay;
+    if (! defined $delay || $delay <= 0) {
+        $delay = 1;
+        $self->loop_delay($delay);
+    }
+    $self->loop_alarm($kernel->alarm_set('session_loop', time() + $delay));
+
     $c->run_hook('engine.end_loop');
 }
 
@@ -165,7 +191,7 @@ sub send_request
 
 sub _poe_start_request
 {
-    my ($kernel, $heap, $request) = @_[KERNEL, HEAP, ARG0];
+    my ($kernel, $self, $heap, $request) = @_[KERNEL, OBJECT, HEAP, ARG0];
     my $c = $heap->{CONTEXT};
 
     # check if this request requires a DNS resolution
@@ -191,7 +217,17 @@ sub _poe_start_request
     }
 
     $c->run_hook('engine.send_request', { request => $request });
-    POE::Kernel->post(&UserAgentAlias, 'request', 'handle_response', $request);
+
+    { # XXX - for debug only...
+        my $uri = $request->uri->clone;
+        $uri->host( $request->notes('original_host') ) if $request->notes('original_host');
+        $c->log->info("Going to fetch $uri");
+    }
+
+    # Choose a random client
+    my @clients = @{ $self->clients };
+    my $client  = $clients[ rand @clients ];
+    POE::Kernel->post($client, 'request', 'handle_response', $request);
 }
 
 sub _poe_got_dns_response
@@ -207,7 +243,7 @@ sub _poe_got_dns_response
 
 sub _poe_handle_response
 {
-    my ($heap, $req_packet, $res_packet) = @_[ HEAP, ARG0, ARG1 ];
+    my ($kernel, $heap, $req_packet, $res_packet) = @_[ KERNEL, HEAP, ARG0, ARG1 ];
 
     my $c = $heap->{CONTEXT};
 
@@ -218,6 +254,9 @@ sub _poe_handle_response
         # Put it back
         $req->uri->host($host);
     }
+    { # XXX - for debug only...
+        $c->log->info("Received " . $req->uri);
+    }
 
     # Work around POE doing too much for us. 
     if (FORCE_ENCODE_CONTENT && $POE::Component::Client::HTTP::VERSION # Hide from CPAN
@@ -247,6 +286,8 @@ sub _poe_handle_response
     }
 
     $c->handle_response($req, $c->prepare_response($res) );
+
+    $kernel->yield('session_loop');
 }
 
 1;
@@ -262,8 +303,9 @@ Gungho::Engine::POE - POE Engine For Gungho
   engine:
     module: POE
     config:
-      loop_delay: 0.5
+      loop_delay: 5 
       client:
+        spawn: 2
         agent:
           - AgentName1
           - AgentName2
@@ -283,11 +325,62 @@ Gungho::Engine::POE - POE Engine For Gungho
 
 Gunghog::Engine::POE gives you the full power of POE to Gungho.
 
-=head1 CHOOSING THE RIGHT loop_delay
+=head1 CONFIGURATION PARAMETERS
+
+You can configure the POE engine in many ways. For convenience, all second 
+level parameter names below are written as 'parent.child'. For example,
+'client.agent' will actually mean
+
+  engine:
+    module: POE
+    config:
+      client:
+        agent: XXXXX
+
+Or in perl,
+
+  engine => {
+    module => 'POE',
+    config => {
+      client => {
+        agent => "XXXX"
+      }
+    }
+  }
+
+=head2 client.loop_delay
 
 C<loop_delay> specifies the number of seconds to wait until calling C<dispatch>
 again. If you feel like Gungho is running slow, try setting this parameter to
-a smaller amount. Things will run much smoother.
+a smaller amount. 
+
+Settings this too low will cause your crawler to be constantly looking up for
+URLs to dispatch instead of fetching the URLs. Alays try to time the requests
+before going to extremes with this setting.
+
+=head2 client.spawn
+
+C<spawn> specifies the number of POE::Component::Client::HTTP sessions to start.
+This will greatly affect your fetching speed, as PoCo::Client::HTTP tends to
+start jamming up after a certain number of requests have been pushed onto
+its queue.
+
+If you feel like all of your other settings are correct but the actual
+HTTP fetch is taking too long, try setting this number to something higher.
+
+By default this is set to 2. 
+
+=head2 keepalive.keep_alive
+
+Specifies the number of seconds to keep a connection in the Keepalive
+connection manager. 
+
+This is an important option to tweak if you're using proxies. Even though
+you might be accessing thousands of different URLs, POE will think that
+you are in fact trying to connect to the same host because you're
+accessing the same proxy.
+
+Turn this to 0 if you are using a proxy.
 
 =head1 POE::Component::Client::HTTP AND DECODED CONTENTS
 
